@@ -7,19 +7,16 @@
  */
 
 
-#include <sys/types.h>
-#include <sys/stat.h>
+#include <dirent.h>     // struct dirent
 #include <fcntl.h>	// AT_ constants (fstatat() flags)
-#include <unistd.h>
-#include <stdio.h>
 
 #include <QMutableListIterator>
 #include <QMultiMap>
 
 #include "DirReadJob.h"
 #include "DirTree.h"
+#include "DirInfo.h"
 #include "DirTreeCache.h"
-#include "Attic.h"
 #include "ExcludeRules.h"
 #include "MountPoints.h"
 #include "Exception.h"
@@ -171,8 +168,14 @@ bool DirReadJob::shouldCrossIntoFilesystem( const DirInfo * dir ) const
 {
     MountPoint * mountPoint = MountPoints::findByPath( dir->url() );
 
+    if ( ! mountPoint )
+    {
+        logError() << "Can't find mount point for " << dir->url() << endl;
+
+        return false;
+    }
+
     bool doCross =
-        mountPoint                     &&
 	! mountPoint->isSystemMount()  &&	//  /dev, /proc, /sys, ...
 	! mountPoint->isDuplicate()    &&	//  bind mount or multiple mounted
 	! mountPoint->isNetworkMount();		//  NFS or CIFS (Samba)
@@ -465,7 +468,7 @@ bool LocalDirReadJob::readCacheFile( const QString & cacheFileName )
 	DirTree * tree = _tree;	 // Copy data members to local variables:
 	DirInfo * dir  = _dir;	 // This object might be deleted soon by killAll()
 
-	if ( _tree->isTopLevel( _dir ) )
+	if ( _tree->isToplevel( _dir ) )
 	{
 	    logDebug() << "Clearing complete tree" << endl;
 
@@ -582,7 +585,7 @@ FileInfo * LocalDirReadJob::stat( const QString & url,
 		parent->insertChild( dir );
 
 	    if ( dir && parent &&
-		 ! tree->isTopLevel( dir ) &&
+		 ! tree->isToplevel( dir ) &&
 		 ! parent->isPkgInfo() &&
 		 dir->device() != parent->device() )
 	    {
