@@ -11,10 +11,10 @@
 #define DirTreeCache_h
 
 
-#include <stdio.h>
-#include <zlib.h>
+#include <zlib.h>    // gzFile
 #include "DirTree.h"
 
+
 #define DEFAULT_CACHE_NAME	".qdirstat.cache.gz"
 #define CACHE_FORMAT_VERSION	"1.0"
 #define MAX_CACHE_LINE_LEN	1024
