@@ -7,12 +7,9 @@
  */
 
 
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <pwd.h>	// getpwuid()
 #include <grp.h>	// getgrgid()
 #include <time.h>       // gmtime()
-#include <unistd.h>
 
 #include <QDateTime>
 
@@ -471,7 +468,7 @@ FileInfo * FileInfo::locate( QString url, bool findPseudoDirs )
                 child = child->next();
             }
 
-            // logDebug() << "Cant find " << url << " in DotEntry" << endl;
+            // logDebug() << "Cannot find " << url << " in DotEntry" << endl;
 	}
 
 	if ( ! result && attic() )
@@ -684,7 +681,7 @@ QString FileInfo::symLinkTarget()
     if ( ! isSymLink() )
         return QString();
 
-    return SysUtil::symLinkTarget( url() );
+    return SysUtil::symLinkTarget( path() );
 }
 
 
@@ -720,6 +717,13 @@ void  FileInfo::processMtime()
 }
 
 
+bool FileInfo::isDominant()
+{
+    return _parent ? _parent->isDominantChild( this ) : false;
+}
+
+
+
 
 //---------------------------------------------------------------------------
 
