@@ -11,9 +11,8 @@
 #define FileInfo_h
 
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <limits.h>
+#include <sys/types.h>  // dev_t, mode_t, nlink_t
+#include <sys/stat.h>   // S_ISDIR() etc.
 
 #include <QTextStream>
 #include <QList>
@@ -62,10 +61,10 @@ namespace QDirStat
      * The most basic building block of a DirTree:
      *
      * Information about one single directory entry. This is the type of info
-     * typically obtained by stat() / lstat() or similar calls.
+     * typically obtained by stat() / lstat() or similar system calls.
      *
      * This class is tuned for size rather than speed: A typical Linux system
-     * easily has 150,000+ filesystem objects, and at least one entry of this
+     * easily has 500,000+ filesystem objects, and at least one entry of this
      * sort is required for each of them.
      *
      * This class provides stubs for children management, yet those stubs all
@@ -73,6 +72,13 @@ namespace QDirStat
      * Derived classes need to take care of that.
      *
      * @short Basic file information (like obtained by the lstat() sys call)
+     *
+     * Important derived classes:
+     *
+     * - DirInfo     for a directory
+     *   - DotEntry  to collect direct file children of a directory
+     *   - Attic     to collect ignored children
+     *   - PkgInfo   for a software package
      **/
     class FileInfo
     {
@@ -515,6 +521,14 @@ namespace QDirStat
 	 **/
 	bool filesystemCanReportBlocks() const;
 
+        /**
+         * Return 'true' if this is a dominant item among its siblings, i.e. if
+         * its total size is much larger than the other items on the same level.
+         *
+         * This forwards the query to the parent, if there is one.
+         **/
+        bool isDominant();
+
 
 	//
 	// Tree management
@@ -534,7 +548,7 @@ namespace QDirStat
 	/**
 	 * Set the "parent" pointer.
 	 **/
-	void setParent( DirInfo *newParent ) { _parent = newParent; }
+	void setParent( DirInfo * newParent ) { _parent = newParent; }
 
 	/**
 	 * Returns a pointer to the next entry on the same level
