@@ -12,13 +12,10 @@
 
 #include "PercentileStats.h"
 #include "FileInfo.h"
-#include "HistogramView.h"
 
 
 namespace QDirStat
 {
-    class DirTree;
-
     /**
      * Helper class for extended file size statistics.
      *
@@ -36,24 +33,10 @@ namespace QDirStat
     public:
 
 	/**
-	 * Constructor.
+	 * Constructor. If 'subtree' is non-null, immediately collect data from
+	 * that subtree.
 	 **/
-	FileSizeStats();
-
-        /**
-         * Populate the internal 'data' list.
-         *
-         * Implemented from PercentileStats.
-         * This overloaded version does not do anything.
-         **/
-        virtual void collect() Q_DECL_OVERRIDE;
-
-	/**
-	 * Recurse through all file elements in the tree and append the own
-	 * size for each file to the data collection. Notice that the data are
-	 * unsorted after this.
-	 **/
-	void collect( DirTree * tree );
+	FileSizeStats( FileInfo * subtree = 0 );
 
 	/**
 	 * Recurse through all file elements in the subtree and append the own
@@ -62,15 +45,6 @@ namespace QDirStat
 	 **/
 	void collect( FileInfo * subtree );
 
-	/**
-	 * Recurse through all file elements in the tree and append the own
-	 * size for each file with the specified suffix to the data
-	 * collection. Notice that the data are unsorted after this.
-	 *
-	 * 'suffix' should start with ".", e.g. ".jpg".
-	 **/
-	void collect( DirTree * tree, const QString & suffix );
-
 	/**
 	 * Recurse through all file elements in the subtree and append the own
 	 * size for each file with the specified suffix to the data
