@@ -7,14 +7,12 @@
  */
 
 
-#include <algorithm>
 #include <QTableWidget>
 #include <QTableWidgetItem>
 #include <QCommandLinkButton>
 #include <QProcess>
 
 #include "FileSizeStatsWindow.h"
-#include "FileSizeStats.h"
 #include "HistogramView.h"
 #include "BucketsTableModel.h"
 #include "DirTree.h"
@@ -138,7 +136,7 @@ void FileSizeStatsWindow::populateSharedInstance( FileInfo *	  subtree,
 						  const QString & suffix  )
 {
     if ( ! subtree )
-        return;
+	return;
 
     sharedInstance()->populate( subtree, suffix );
     sharedInstance()->show();
@@ -205,23 +203,24 @@ QStringList FileSizeStatsWindow::quantile( int order, const QString & name )
 }
 
 
-void FileSizeStatsWindow::fillQuantileTable( QTableWidget *    table,
-					     int	       order,
-					     const QString &   namePrefix,
-					     const QRealList & sums,
-					     int	       step,
-					     int	       extremesMargin )
+void FileSizeStatsWindow::fillQuantileTable( QTableWidget *	    table,
+					     int		    order,
+					     const QString &	    namePrefix,
+					     const PercentileSums & sums,
+					     int		    step,
+					     int		    extremesMargin )
 {
     enum TableColumns
     {
 	NumberCol,
 	ValueCol,
 	NameCol,
-	SumCol
+	SumCol,
+	CumulativeSumCol
     };
 
     table->clear();
-    table->setColumnCount( sums.isEmpty() ? 3 : 4 );
+    table->setColumnCount( sums.isEmpty() ? 3 : 5 );
     table->setRowCount( order + 1 );
 
     QStringList header;
@@ -237,7 +236,10 @@ void FileSizeStatsWindow::fillQuantileTable( QTableWidget *    table,
     header << tr( "Value" ) << tr( "Name" );
 
     if ( ! sums.isEmpty() )
+    {
 	header << tr( "Sum %1(n-1)..%2(n)" ).arg( namePrefix ).arg( namePrefix );
+	header << tr( "Cumulative Sum" );
+    }
 
     for ( int col = 0; col < header.size(); ++col )
     {
@@ -269,8 +271,11 @@ void FileSizeStatsWindow::fillQuantileTable( QTableWidget *    table,
 	addItem( table, row, NumberCol, namePrefix + QString::number( i ) );
 	addItem( table, row, ValueCol, formatSize( _stats->quantile( order, i ) ) );
 
-	if ( i > 0 && i < sums.size() )
-	    addItem( table, row, SumCol, formatSize( sums.at( i ) ) );
+    if ( i > 0 && i < sums.size() )
+    {
+	addItem( table, row, SumCol, formatSize( sums.individual().at( i ) ) );
+	addItem( table, row, CumulativeSumCol, formatSize( sums.cumulative().at( i ) ) );
+    }
 
 	if ( i == 0 || i == median || i == order || i == quartile_1 || i == quartile_3 )
 	{
@@ -301,6 +306,7 @@ void FileSizeStatsWindow::fillQuantileTable( QTableWidget *    table,
     setColAlignment( table, ValueCol,  Qt::AlignRight  | Qt::AlignVCenter );
     setColAlignment( table, NameCol,   Qt::AlignCenter | Qt::AlignVCenter );
     setColAlignment( table, SumCol,    Qt::AlignRight  | Qt::AlignVCenter );
+    setColAlignment( table, CumulativeSumCol,	 Qt::AlignRight	 | Qt::AlignVCenter );
 
     HeaderTweaker::resizeToContents( table->horizontalHeader() );
 }
@@ -379,7 +385,7 @@ void FileSizeStatsWindow::fillHistogram()
 
     histogram->clear();
     histogram->setPercentiles( _stats->percentileList() );
-    histogram->setPercentileSums( _stats->percentileSums() );
+    histogram->setPercentileSums( _stats->percentileSums().individual() );
     histogram->autoStartEndPercentiles();
     updateOptions();
     fillBuckets();
