@@ -8,14 +8,17 @@
 
 
 #include <QApplication>
-#include <QCloseEvent>
-#include <QMouseEvent>
-#include <QMessageBox>
-#include <QFileDialog>
 #include <QClipboard>
+#include <QCloseEvent>
+#include <QFileDialog>
+#include <QFileInfo>
+#include <QMessageBox>
+#include <QMouseEvent>
 
 #include "MainWindow.h"
 #include "ActionManager.h"
+#include "BookmarksManager.h"
+#include "BusyPopup.h"
 #include "CleanupCollection.h"
 #include "CleanupConfigPage.h"
 #include "ConfigDialog.h"
@@ -27,8 +30,10 @@
 #include "Exception.h"
 #include "ExcludeRules.h"
 #include "FileDetailsView.h"
+#include "FileSearchFilter.h"
 #include "FileSizeStatsWindow.h"
 #include "FileTypeStatsWindow.h"
+#include "FindFilesDialog.h"
 #include "Logger.h"
 #include "MimeCategorizer.h"
 #include "OpenDirDialog.h"
@@ -42,6 +47,7 @@
 #include "SelectionModel.h"
 #include "Settings.h"
 #include "SettingsHelpers.h"
+#include "SignalBlocker.h"
 #include "SysUtil.h"
 #include "Trash.h"
 #include "UnreadableDirsWindow.h"
@@ -77,7 +83,6 @@ MainWindow::MainWindow():
     _updateTimer.setInterval( UPDATE_MILLISEC );
     _treeExpandTimer.setSingleShot( true );
     _dUrl = _ui->actionDonate->iconText();
-    _futureSelection.setUseRootFallback( false );
     _ui->menubar->setCornerWidget( new QLabel( MENUBAR_VERSION ) );
 
     // The first call to app() creates the QDirStatApp and with it
@@ -92,6 +97,27 @@ MainWindow::MainWindow():
     _ui->treemapView->setDirTree( app()->dirTree() );
     _ui->treemapView->setSelectionModel( app()->selectionModel() );
 
+    _futureSelection.setTree( app()->dirTree() );
+    _futureSelection.setUseParentFallback( true );
+
+    app()->bookmarksManager()->setBookmarksMenu( _ui->menuBookmarks );
+    app()->bookmarksManager()->read();
+    app()->bookmarksManager()->rebuildBookmarksMenu();
+
+
+    // Set the boldItemFont for the DirTreeModel.
+    //
+    // It can't fetch that by itself from the DirTreeView;
+    // we'd get an initialization sequence problem.
+    // So this has to be done from the outside when both are created.
+
+    QFont boldItemFont = _ui->dirTreeView->font();
+    boldItemFont.setWeight( QFont::Bold );
+    app()->dirTreeModel()->setBoldItemFont( boldItemFont );
+
+
+    // Initialize cleanups
+
     app()->cleanupCollection()->addToMenu   ( _ui->menuCleanup,
                                               true ); // keepUpdated
     app()->cleanupCollection()->addToToolBar( _ui->toolBar,
@@ -140,6 +166,7 @@ MainWindow::~MainWindow()
     writeSettings();
     ExcludeRules::instance()->writeSettings();
     MimeCategorizer::instance()->writeSettings();
+    app()->bookmarksManager()->write();
 
     // Relying on the QObject hierarchy to properly clean this up resulted in a
     //	segfault; there was probably a problem in the deletion order.
@@ -210,9 +237,18 @@ void MainWindow::connectSignals()
     connect( app()->selectionModel(),	 SIGNAL( currentItemChanged( FileInfo *, FileInfo * ) ),
 	     _historyButtons,		 SLOT  ( addToHistory	   ( FileInfo *		    ) ) );
 
+    connect( app()->selectionModel(),	 SIGNAL( currentItemChanged  ( FileInfo *, FileInfo * ) ),
+	     this,      		 SLOT  ( updateBookmarkButton( FileInfo *             ) ) );
+
     connect( _historyButtons,		 SIGNAL( navigateToUrl( QString ) ),
 	     this,			 SLOT  ( navigateToUrl( QString ) ) );
 
+    connect( app()->bookmarksManager(),  SIGNAL( navigateToUrl( QString ) ),
+             this,                       SLOT  ( navigateToUrl( QString ) ) );
+
+    connect( _ui->bookmarkButton,        SIGNAL( toggled            ( bool ) ),
+             this,                       SLOT  ( bookmarkCurrentPath( bool ) ) );
+
     connect( _ui->breadcrumbNavigator,	 SIGNAL( pathClicked   ( QString ) ),
 	     app()->selectionModel(),	 SLOT  ( setCurrentItem( QString ) ) );
 
@@ -229,7 +265,7 @@ void MainWindow::connectSignals()
 	     this,			 SLOT  ( showElapsedTime() ) );
 
     connect( &_treeExpandTimer,		  SIGNAL( timeout() ),
-	     _ui->actionExpandTreeLevel1, SLOT( trigger()   ) );
+	     _ui->actionExpandTreeLevel1, SLOT  ( trigger()   ) );
 
     if ( _useTreemapHover )
     {
@@ -262,7 +298,7 @@ void MainWindow::updateActions()
 
     _ui->actionCopyPathToClipboard->setEnabled( currentItem );
     _ui->actionGoUp->setEnabled( currentItem && currentItem->treeLevel() > 1 );
-    _ui->actionGoToToplevel->setEnabled( firstToplevel && ( ! currentItem || currentItem->treeLevel() > 1 ));
+    _ui->actionGoToToplevel->setEnabled( firstToplevel );
 
     FileInfoSet selectedItems = app()->selectionModel()->selectedItems();
     FileInfo * sel	      = selectedItems.first();
@@ -442,8 +478,9 @@ void MainWindow::idleDisplay()
     int sortCol = QDirStat::DataColumns::toViewCol( QDirStat::PercentNumCol );
     _ui->dirTreeView->sortByColumn( sortCol, Qt::DescendingOrder );
 
-    if ( ! _futureSelection.isEmpty() )
+    if ( _futureSelection.subtree() )
     {
+        // logDebug() << "Using future selection " << _futureSelection.subtree() << endl;
         _treeExpandTimer.stop();
         applyFutureSelection();
     }
@@ -477,6 +514,15 @@ void MainWindow::updateFileDetailsView()
 }
 
 
+void MainWindow::setDetailsPanelVisible( bool visible )
+{
+    _ui->fileDetailsPanel->setVisible( visible );
+
+    if ( visible )
+        updateFileDetailsView();
+}
+
+
 void MainWindow::startingReading()
 {
     _stopWatch.start();
@@ -529,12 +575,20 @@ void MainWindow::openUrl( const QString & url )
 }
 
 
-void MainWindow::openDir( const QString & url )
+void MainWindow::openDir( const QString & origUrl )
 {
+    QString url = handleSymLink( origUrl );
+
     try
     {
+        if ( url.startsWith( "/" ) )
+            _futureSelection.setUrl( url );
+        else
+            _futureSelection.clear();
+
 	app()->dirTreeModel()->openUrl( url );
 	updateWindowTitle( app()->dirTree()->url() );
+        app()->bookmarksManager()->setBaseUrl( app()->dirTree()->url() );
     }
     catch ( const SysCallFailedException & ex )
     {
@@ -563,6 +617,53 @@ void MainWindow::showOpenDirErrorPopup( const SysCallFailedException & ex )
 }
 
 
+QString MainWindow::handleSymLink( const QString & origUrl ) const
+{
+    QString   url( origUrl );
+    QFileInfo urlInfo( url );
+
+    if ( urlInfo.isSymLink() )
+    {
+        PanelMessage * msg = 0;
+
+        if ( urlInfo.exists() )
+        {
+            QString target = urlInfo.canonicalFilePath();
+
+            logInfo() << "Following symlink \"" << url
+                      <<"\" to target \"" << target << "\"" << endl;
+
+            msg = new PanelMessage( _ui->messagePanel );
+            CHECK_NEW( msg );
+
+            msg->setHeading( tr( "Following symbolic link" ) );
+            msg->setText( tr( "%1 is a symbolic link to %2" )
+                          .arg( url ).arg( target ) );
+            _ui->messagePanel->add( msg );
+
+            url = target;
+        }
+        else
+        {
+            logError() << "Broken symlink";
+
+            msg = new PanelMessage( _ui->messagePanel );
+            CHECK_NEW( msg );
+
+            msg->setHeading( tr( "Broken symbolic link" ) );
+            msg->setText( tr( "%1 is a broken symbolic link to %2" )
+                          .arg( url ).arg( urlInfo.symLinkTarget() ) );
+            msg->setIcon( QPixmap( ":/icons/dialog-warning.png" ) );
+        }
+
+        if ( msg )
+            _ui->messagePanel->add( msg );
+    }
+
+    return url;
+}
+
+
 void MainWindow::askOpenDir()
 {
     QString path;
@@ -598,13 +699,33 @@ void MainWindow::askOpenPkg()
 }
 
 
+void MainWindow::askFindFiles()
+{
+    bool canceled;
+    FileSearchFilter filter = FindFilesDialog::askFindFiles( &canceled );
+
+    if ( ! canceled )
+    {
+	_discoverActions->findFiles( filter );
+    }
+}
+
+
 void MainWindow::readPkg( const PkgFilter & pkgFilter )
 {
-    // logInfo() << "URL: " << pkgFilter.url() << endl;
+    logInfo() << "URL: " << pkgFilter.url() << endl;
 
+    _futureSelection.setUrl( "Pkg:/" );
     updateWindowTitle( pkgFilter.url() );
-    expandTreeToLevel( 0 );   // Performance boost: Down from 25 to 6 sec.
+    _ui->breadcrumbNavigator->clear();
+    _ui->fileDetailsView->clear();
+    app()->dirTreeModel()->clear();
+
+    BusyPopup msg( tr( "Reading package database..." ), this );
+
     app()->dirTreeModel()->readPkg( pkgFilter );
+    app()->bookmarksManager()->setBaseUrl( app()->dirTree()->url() );
+    app()->selectionModel()->setCurrentItem( app()->dirTree()->firstToplevel() );
 }
 
 
@@ -616,6 +737,7 @@ void MainWindow::refreshAll()
     if ( ! url.isEmpty() )
     {
 	logDebug() << "Refreshing " << url << endl;
+        _futureSelection.setUrl( url );
 
 	if ( PkgFilter::isPkgUrl( url ) )
 	    app()->dirTreeModel()->readPkg( url );
@@ -642,7 +764,7 @@ void MainWindow::refreshSelected()
 {
     busyDisplay();
     _futureSelection.set( app()->selectionModel()->selectedItems().first() );
-    // logDebug() << "Setting future selection: " << _futureSelection.subtree() << endl;
+    // logDebug() << "Setting future selection: " << _futureSelection.url() << endl;
     app()->dirTreeModel()->refreshSelected();
     updateActions();
 }
@@ -650,16 +772,26 @@ void MainWindow::refreshSelected()
 
 void MainWindow::applyFutureSelection()
 {
-    FileInfo * sel = _futureSelection.subtree();
-    // logDebug() << "Using future selection: " << sel << endl;
+    FileInfo * sel    = _futureSelection.subtree();
+    DirInfo  * branch = _futureSelection.dir();
+    _futureSelection.clear();
+
+#if 0
+    logDebug() << "Using future selection: " << sel << endl;
+    logDebug() << "Branch: " << branch << endl;
+#endif
 
     if ( sel )
     {
         _treeExpandTimer.stop();
-        _futureSelection.clear();
-        app()->selectionModel()->setCurrentBranch( sel );
 
-        if ( sel->isMountPoint() )
+        if ( branch )
+            app()->selectionModel()->setCurrentBranch( branch );
+
+        app()->selectionModel()->setCurrentItem( sel,
+                                                 true);  // select
+
+        if ( sel->isMountPoint() || sel->isDirInfo() )  // || app()->dirTree()->isToplevel( sel ) )
             _ui->dirTreeView->setExpanded( sel, true );
     }
 }
@@ -790,12 +922,21 @@ void MainWindow::showSummary()
 
 void MainWindow::startingCleanup( const QString & cleanupName )
 {
+    // Notice that this is not called for actions that are not owned by the
+    // CleanupCollection such as _ui->actionMoveToTrash().
+
+    FileInfo * sel = app()->selectionModel()->selectedItems().first();
+    _futureSelection.set( sel );
+    logDebug() << "Storing future selection " << sel << endl;
     showProgress( tr( "Starting cleanup action %1" ).arg( cleanupName ) );
 }
 
 
 void MainWindow::cleanupFinished( int errorCount )
 {
+    // Notice that this is not called for actions that are not owned by the
+    // CleanupCollection such as _ui->actionMoveToTrash().
+
     logDebug() << "Error count: " << errorCount << endl;
 
     if ( errorCount == 0 )
@@ -844,11 +985,23 @@ void MainWindow::navigateUp()
 {
     FileInfo * currentItem = app()->selectionModel()->currentItem();
 
-    if ( currentItem && currentItem->parent() &&
-	 currentItem->parent() != app()->dirTree()->root() )
+    if ( currentItem )
     {
-	app()->selectionModel()->setCurrentItem( currentItem->parent(),
-					 true ); // select
+        FileInfo * parent = currentItem->parent();
+
+        if ( parent && parent != app()->dirTree()->root() )
+        {
+
+            // Close and re-open the parent to enforce a screen update:
+            // Sometimes the bold font is not taken into account when moving
+            // upwards, and so every column is cut off (probably a Qt bug)
+            _ui->dirTreeView->setExpanded( parent, false );
+
+            app()->selectionModel()->setCurrentItem( parent,
+                                                     true ); // select
+            // Re-open the parent
+            _ui->dirTreeView->setExpanded( parent, true );
+        }
     }
 }
 
@@ -886,10 +1039,90 @@ void MainWindow::navigateToUrl( const QString & url )
 }
 
 
+void MainWindow::updateBookmarkButton( FileInfo * newCurrent )
+{
+    SignalBlocker sigBlocker( _ui->bookmarkButton );  // Prevent signal ping-pong
+
+    if ( ! newCurrent )  // No selection / no current item
+    {
+        // logDebug() << "No current item" << endl;
+        _ui->bookmarkButton->setChecked( false );
+    }
+    else
+    {
+        DirInfo * currentDir = newCurrent->toDirInfo();
+
+        if ( ! currentDir && newCurrent->parent() )
+            currentDir = newCurrent->parent();
+
+        if ( currentDir )
+        {
+            QString url = currentDir->debugUrl();
+            bool isBookmarked = app()->bookmarksManager()->contains( url );
+            _ui->bookmarkButton->setChecked( isBookmarked );
+#if 0
+            if ( isBookmarked )
+                logDebug() << url << " is bookmarked" << endl;
+#endif
+        }
+    }
+}
+
+
+void MainWindow::bookmarkCurrentPath( bool isChecked )
+{
+    //
+    // Find out the current item and directory
+    //
+
+    FileInfo * currentItem = app()->selectionModel()->currentItem();
+
+    if ( ! currentItem )
+        return;
+
+    DirInfo * currentDir = currentItem->toDirInfo();
+
+    if ( ! currentDir && currentItem->parent() )  // Not a directory?
+        currentDir = currentItem-> parent();      // Move up one level
+
+    if ( currentDir )
+    {
+        //
+        // Add or remove it from the bookmarks
+        //
+
+        // debugUrl() because we want the <Files> DotEntry if there is one
+        QString url = currentDir->debugUrl();
+
+        if ( isChecked )
+        {
+            app()->bookmarksManager()->add( url );
+            // The BookmarksManager is already logging this
+            showProgress( tr( "Bookmarked %1" ).arg( url ) );
+        }
+        else
+        {
+            app()->bookmarksManager()->remove( url );
+            // The BookmarksManager is already logging this
+            showProgress( tr( "Un-bookmarked %1" ).arg( url ) );
+        }
+    }
+}
+
+
 void MainWindow::moveToTrash()
 {
+    // _ui->actionMoveToTrash() is not a subclass of Cleanup and not owned by
+    // CleanupCollection, so this has to replicate some of its functionality.
+
     FileInfoSet selectedItems = app()->selectionModel()->selectedItems().normalized();
 
+    // Save the selection - at least the first selected item
+
+    FileInfo * sel = selectedItems.first();
+    _futureSelection.set( sel );
+    logDebug() << "Storing future selection " << sel << endl;
+
     // Prepare output window
 
     OutputWindow * outputWindow = new OutputWindow( qApp->activeWindow() );
