@@ -178,6 +178,11 @@ public slots:
      **/
     void updateFileDetailsView();
 
+    /**
+     * Show or hide the details view.
+     **/
+    void setDetailsPanelVisible( bool visible );
+
     /**
      * Copy the path of the current item (if there is one) to the system
      * clipboard for use in other applications.
@@ -189,6 +194,11 @@ public slots:
      **/
     void moveToTrash();
 
+    /**
+     * Open the "Find Files" dialog and display the results.
+     **/
+    void askFindFiles();
+
     /**
      * Navigate one directory level up.
      **/
@@ -297,6 +307,18 @@ protected slots:
      **/
     void navigateToUrl( const QString & url );
 
+    /**
+     * Notification that the current item changed: Find out the new current
+     * directory, check if that is in the bookmarks collection, and change the
+     * on/off status (i.e. the icon) of the bookmarksButton accordingly.
+     **/
+    void updateBookmarkButton( FileInfo * newCurrent );
+
+    /**
+     * Bookmark or un-bookmark the current directory.
+     **/
+    void bookmarkCurrentPath( bool isChecked );
+
     /**
      * Open the config dialog.
      **/
@@ -471,10 +493,19 @@ protected:
      * Show an error popup that a directory could not be opened and wait until
      * the user confirmed it.
      *
-     * The relevant informatoin is all in the exception.
+     * The relevant information is all in the exception.
      **/
     void showOpenDirErrorPopup( const SysCallFailedException & ex );
 
+    /**
+     * Handle a symlink as an argument to reading a new directory tree
+     * e.g. in openDir(): Follow the symlink and return the target URL.
+     *
+     * If it is a valid symlink, post a PanelMessage to inform the user about
+     * it. If it's a broken symlink, post an error as a PanelMessage.
+     **/
+    QString handleSymLink( const QString & origUrl ) const;
+
     /**
      * Handle mouse buttons: Activate history actions actionGoBack and
      * actionGoForward with the "back" and "forward" mouse buttons as well.
@@ -511,7 +542,10 @@ private:
 
 
 /**
- * Helper class for the different layouts of the tree view layout.
+ * Helper class for the different layouts of the tree view layout that
+ * correspond to the [L1], [L2], [L3] buttons in the tool bar where you can
+ * switch what columns are displayed, and whether or not to display the
+ * details panel.
  *
  * Notice that the column layouts are handled in the HeaderTweaker and its
  * ColumnLayout helper class; see also HeaderTweaker.h and HeaderTweaker.cpp.
@@ -519,14 +553,13 @@ private:
 class TreeLayout
 {
 public:
+
     TreeLayout( const QString & name ):
 	name( name ),
-	showCurrentPath( true ),
 	showDetailsPanel( true )
 	{}
 
     QString name;
-    bool    showCurrentPath;
     bool    showDetailsPanel;
 
 }; // class TreeLayout
