@@ -14,6 +14,9 @@
 
 #include "MimeCategory.h"
 
+#define CATEGORY_EXECUTABLES "Executables"
+#define CATEGORY_SYMLINKS    "Symlinks"
+
 
 namespace QDirStat
 {
@@ -56,6 +59,12 @@ namespace QDirStat
 	 **/
 	static MimeCategorizer * instance();
 
+	/**
+	 * Return the color for a FileInfo item or white if it doesn't fit
+	 * into any of the available categories.
+	 **/
+	QColor color( FileInfo * item );
+
 	/**
 	 * Return the MimeCategory for a FileInfo item or 0 if it doesn't fit
 	 * into any of the available categories.
@@ -129,12 +138,23 @@ namespace QDirStat
 			  MimeCategory			* category,
 			  const QStringList		& suffixList  );
 
+	/**
+	 * Iterate over all categories to find categories by name.
+	 **/
+	MimeCategory * matchCategoryName( const QString & categoryName ) const;
+
 	/**
 	 * Iterate over all categories and try all patterns until the first
 	 * match. Return the matched category or 0 if none matched.
 	 **/
 	MimeCategory * matchPatterns( const QString & filename ) const;
 
+	/**
+	 * Make sure that the Executable and Symlink categories exist, in case
+	 * they have been manually removed from the configuration file.
+	 **/
+	void ensureMandatoryCategories();
+
 	/**
 	 * Add default categories in case none were read from the settings.
 	 **/
@@ -152,6 +172,9 @@ namespace QDirStat
 	QMap<QString, MimeCategory *>	_caseInsensitiveSuffixMap;
 	QMap<QString, MimeCategory *>	_caseSensitiveSuffixMap;
 
+	MimeCategory *_executableCategory;
+	MimeCategory *_symlinkCategory;
+
     };	// class MimeCategorizer
 
 }	// namespace QDirStat
