@@ -345,6 +345,8 @@ bool MountPoints::read( const QString & filename )
     }
 
     findNtfsDevices();
+    logDebug() << "Reading " << filename << endl;
+
     QTextStream in( &file );
     int lineNo = 0;
     int count  = 0;
@@ -387,6 +389,9 @@ bool MountPoints::read( const QString & filename )
         postProcess( mountPoint );
         add( mountPoint );
 
+        if ( ! mountPoint->isDuplicate() )
+            ++count;
+
 	line = in.readLine();
     }
 
