@@ -11,6 +11,7 @@
 #define TreeWalker_h
 
 #include "FileInfo.h"
+#include "FileSearchFilter.h"
 
 
 namespace QDirStat
@@ -36,7 +37,10 @@ namespace QDirStat
     {
     public:
 
-        TreeWalker() {}
+        TreeWalker():
+            _overflow( false )
+            {}
+
         virtual ~TreeWalker() {}
 
         /**
@@ -59,6 +63,15 @@ namespace QDirStat
          **/
         virtual bool check( FileInfo * item ) = 0;
 
+        /**
+         * Flag: Results overflow while walking the tree?
+         *
+         * Derived classes can use this to indicate that the number of results
+         * was limited.
+         **/
+        bool overflow() const { return _overflow; }
+
+
     protected:
 
         /**
@@ -73,6 +86,13 @@ namespace QDirStat
          **/
         qreal lowerPercentileThreshold( PercentileStats & stats );
 
+
+        //
+        // Data members
+        //
+
+        bool _overflow;
+
     };  // class TreeWalker
 
 
@@ -221,6 +241,27 @@ namespace QDirStat
         short _month;
     };
 
+
+    /**
+     * TreeWalker to find files and/or directories that match a pattern.
+     **/
+    class FindFilesTreeWalker: public TreeWalker
+    {
+    public:
+        FindFilesTreeWalker( const FileSearchFilter & filter ):
+            TreeWalker(),
+            _filter( filter ),
+            _count( 0 )
+            {}
+
+        virtual bool check( FileInfo * item );
+
+    protected:
+
+        FileSearchFilter _filter;
+        int              _count;
+    };
+
 }       // namespace QDirStat
 
 #endif  // TreeWalker_h
